package javaxml;

import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigInteger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author jjoller;
 * @version 1.0
 *
 * Einfaches Servlet, welches ein Eingabe einer Integer Zahl die nchst mgliche Primzahl liefert.
 * Created on 08.08.2003
 *
 * Beispiel:
 * http://localhost:8080/JavaXML3/servlet/javaxml.PrimzahlServlet?12
 * oder
 * http://localhost:8080/JavaXML3/servlet/javaxml.PrimzahlServlet?startZahl=345&&debug=true
 * oder
 * http://localhost:8080/JavaXML3/servlet/javaxml.PrimzahlServlet?startZahl=5486
 */


public class PrimzahlServlet extends HttpServlet {

	/**
	 * GET Befehl
	 * @param Request und Response 
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {
		System.out.println("[PrimzahlServlet]doGet()");
		/**
		 * Lesen der Eingabezahl
		 */
		int iStart = 10;
		boolean bDebug = false;
		String strStart = request.getParameter("startZahl");
		String strDebug = "false";
		strDebug = request.getParameter("debug");
		if (strDebug == null)
			strDebug = "false";
		try {
			iStart = Integer.parseInt(strStart);
		} catch (Exception e) { // NumberFormat oder NullPointerException
			// somit wird iStart Standardwert verwendet 
		}
		System.out.println("[PrimzahlServlet.doGet()]Eingabe \t" + iStart);
		System.out.println("[PrimzahlServlet.doGet()]        \t" + strDebug);
		bDebug = strDebug.equalsIgnoreCase("true");

		System.out.println(
			"[PrimzahlServlet.doGet()]generieren der Antwort in XML");
		/**
		 * MIME Type
		 */
		response.setContentType("text/xml; charset=UTF-8");
		PrintWriter pw = response.getWriter();

		/**
		 * XML Header
		 */
		pw.println("<?xml version=\"1.0\"?>");
		/**
		 * Formattierung der Ausgabe mit CSS (muss ich noch erstellen!)
		 */
		//pw.print  ("<?xml-stylesheet ");
		//pw.println("type='text/css' href='styles/primzahlen.css'?>");  

		/**
		 * Bestimmen der Primzahl zu iStart ohne Debug Info-Ausgabe
		 */
		int iPrime = this.getPrime(iStart, bDebug);
		pw.println("<PrimZahlen>");

		/**
		 * Ausgabe der Primzahl 
		 */
		pw.print("  <primzahl Startzahl=\"" + iStart + "\">");
		pw.print(iPrime);

		/**
		 * End-Tags
		 */
		pw.println("</primzahl>");
		pw.println("</PrimZahlen>");
		pw.close();
	}

	/**
	 * doPost sendet die Antwort an den Client bei POST Anfragen
	 * Hier ruft die Methode einfach doGet() auf.
	 */
	public void doPost(
		HttpServletRequest request,
		HttpServletResponse response)
		throws ServletException, IOException {

		System.out.println("[PrimzahlServlet]doGet()");
		this.doGet(request, response);

	}
	public int getPrime(int i, boolean debug) {
		System.out.println(
			"[PrimzahlServlet]getPrime(" + i + "," + debug + ")");
		// solange i keine Primzahl ist, addiere 1
		while (!BigInteger.valueOf(i).isProbablePrime(20)) {
			if (debug)
				System.out.println("Primzahlsuche : " + BigInteger.valueOf(i));
			i++;
		}
		// wahrscheinlich ist i jetzt Primzahl
		// (mit Wahrscheinlichkeit 1- 1/(2^20) ~= 1 zu einer Million)
		return i;
	}

}
