/*
 * Created on 07.08.2003
 *
 */
package javaxml;

/**
 * Beispiel 3 : GenericHTTPSoapClient mit Tunnel (mit Servlet SimpleHTTPReceive)
 * @author jjoller
 */
/**
 *	Das Beispiel zeigt, wie man den Apache Tunnel einsetzen knnte.
 *	Dabei werden die Calls an einen andern Port umgeleitet.
 *	In der Tunnel Applikation sieht man den Client und den Server.
 *	Der Tunnel wird wiefolgt gestartet: javaw org.apache.soap.util.net.TcpTunnelGui 5555 localhost 8080
 */

/**
*
* usage:
* 	java HttpClient (options)
* 		options:
* 		  -url hosturl      Specify host url
* 							Default host url: DEFAULT_HOST_URL
* 		  -df datafile      Specify data file name
* 							Default host file name: DEFAULT_DATA_FILENAME
* 		  -h                This help screen.
* 		  -at               Specify Attachment file name
*
* 
*/

import java.io.*;
import java.util.*;

public class SOAPPrimeClient
{
	/**
	 * Standardwerte fr die Parameter
	 */
	/**
	 * Host URL
	 */
	private static final String DEFAULT_HOST_URL    = "http://localhost:5555/JavaXML3/servlet/javaxml.SOAPPrimzahlServlet";
	/**
	 * Dateiname (XML Datei: PO.xml)
	 */
	private static final String DEFAULT_DATA_FILENAME   = "SOAPGetPrime.xml";
	/**
	 * Namespace
	 */
	private static final String URI                     = "urn:javaxml";
	/**
	 * Member Variablen
	 */
	/**
	 * m_hostURL kann als Parameter den obigen Standardwert berschreiben
	 */
	private String m_hostURL;
	/**
	 * m_dataFileName kann den obigen Standardwert berschreiben
	 */
	private String m_dataFileName;

	/**
 	* Konstruktor
 	* @param hostURL URL auf das Servlet : http://localhost:5555/JavaXML3/servlet/javaxml.PrimzahlServlet
 	* @param dataFileName XML Dokument : PO.XML
 	* @throws Exception
 	*/
	public SOAPPrimeClient(String hostURL, String dataFileName) throws Exception
	{
		System.out.println("[SOAPPrimeClient]SOAPPrimeClient(\""+hostURL+"\" , \""+dataFileName+"\")");
		System.out.println("[SOAPPrimeClient]SOAP Client mit SOAP Header");
		m_hostURL = hostURL;
		m_dataFileName    = dataFileName;

		System.out.println();
		System.out.println("_________________________________________________________");
		System.out.println("Starting SOAPPrimeClient:");
		System.out.println("    host url        = " + m_hostURL);
		System.out.println("    data file       = " + m_dataFileName);
		System.out.println("___________________________________________________________");
		System.out.println();
	}

	/**
	 * Senden einer SOAP Message, ohne Attachment
	 * Die Message wird aus dem oben angegebenen File gelesen
	 *
	 */
	public void sendSOAPMessage()
	{
		System.out.println("[SOAPPrimeClient]sendSOAPMessage()");
		try
		{
			// SOAP body fr den SOAP envelope
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]FileReader");
			FileReader fr = new FileReader (m_dataFileName);
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]parsers.DocumentBuilder");
			javax.xml.parsers.DocumentBuilder xdb =
				org.apache.soap.util.xml.XMLParserUtils.getXMLDocBuilder();
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]dom.Document");
			org.w3c.dom.Document doc =
				xdb.parse (new org.xml.sax.InputSource (fr));
			if (doc == null) {
				throw new org.apache.soap.SOAPException
					(org.apache.soap.Constants.FAULT_CODE_CLIENT, "parsing error");
			}

			// Vector fr die Header Elemente
			Vector headerElements = new Vector();

			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]domElement header mustunderstand");
			// Header Elemente in einem Namespace
			org.w3c.dom.Element headerElement =
				doc.createElementNS(URI,"javaxml:MessageHeader");

			headerElement.setAttributeNS(URI,"SOAP-ENV:mustUnderstand","1");

			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]domElement Sub-Nodes des headers (from, to)");
			// Subnodes im MessageHeader
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]from Element");
			org.w3c.dom.Element ele = doc.createElement("From");
			org.w3c.dom.Text textNode = doc.createTextNode("From_Client");
			org.w3c.dom.Node tempNode = ele.appendChild(textNode);

			tempNode = headerElement.appendChild(ele);

			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]to Element");
			ele = doc.createElement("To");
			textNode = doc.createTextNode("To_Client");
			tempNode = ele.appendChild(textNode);

			tempNode = headerElement.appendChild(ele);

			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]MessageId");
			ele = doc.createElement("MessageId");
			textNode = doc.createTextNode("9999");
			tempNode = ele.appendChild(textNode);

			tempNode = headerElement.appendChild(ele);

			headerElements.add(headerElement);

			// create a vector for collecting the body elements
			Vector bodyElements = new Vector();
			//parse xml elemnt as soap body element
			bodyElements.add(doc.getDocumentElement ());

			//Create the SOAP envelope
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]soap.Envelope");
			org.apache.soap.Envelope envelope = new org.apache.soap.Envelope();

			//Add the SOAP header element to the envelope
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]soap.Header");
			org.apache.soap.Header header = new org.apache.soap.Header();
			header.setHeaderEntries(headerElements);
			envelope.setHeader(header);

			//Create the SOAP body element
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]soap.Body");
			org.apache.soap.Body body = new org.apache.soap.Body();
			body.setBodyEntries(bodyElements);
			//Add the SOAP body element to the envelope
			envelope.setBody(body);

			// Build the Message.
			org.apache.soap.messaging.Message msg =
				new org.apache.soap.messaging.Message();

			msg.send (new java.net.URL(m_hostURL), URI, envelope);
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]Sende SOAP Message mit Apache HTTP SOAP Client.");

			// receive response from the transport and dump it to the screen
			System.out.println("[SOAPPrimeClient.sendSOAPMessage()]Warten auf Antwort....");
			org.apache.soap.transport.SOAPTransport st = msg.getSOAPTransport ();
			BufferedReader br = st.receive ();
			String line = br.readLine();
			if(line == null)
			{
				System.out.println("[SOAPPrimeClient.sendSOAPMessage()]HTTP POST war erfolgreich. \n");
			}
			else
			{
			   while (line != null)
			   {
					System.out.println (line);
					line = br.readLine();
			   }
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	//
	// NOTE: the remainder of this deals with reading arguments
	//
	/** GenericHTTPSoapClient_MitTunnel. */
	public static void main(String args[]) {



		// Values to be read from parameters
		String hostURL    	    = DEFAULT_HOST_URL;
		String dataFileName     = DEFAULT_DATA_FILENAME;
		String attachment       = null;

		// Check parameters
		for (int i = 0; i < args.length; i++) {
			String arg = args[i];
			System.out.println("\tEingabe-Parameter["+i+"] : "+arg);
			// Options
			if (!arg.startsWith("-")) {
				System.err.println ("error: unexpected argument - "+arg);
				printUsage();
				System.exit(1);
			}
			else {
				if (arg.equals("-url")) {
					if (i == args.length - 1 || args[i+1].startsWith("-")) {
						System.err.println("error: missing host address parameter");
						System.exit(1);
					}
					hostURL = args[++i];
					continue;
				}
				if (arg.equals("-df")) {
					if (i == args.length - 1 || args[i+1].startsWith("-")) {
						System.err.println("error: missing data file parameter");
						System.exit(1);
					}
					dataFileName = args[++i];
					continue;
				}if (arg.equals("-at")) {
					if (i == args.length - 1 || args[i+1].startsWith("-")) {
						System.err.println("error: missing attachment parameter");
						System.exit(1);
					}
					attachment = args[++i];
					continue;
				}

				if (arg.equals("-h")) {
					printUsage();
					System.exit(1);
				}
			}
		}

		// Start the HTTPSoapClient
		try
		{
			SOAPPrimeClient soapClient =
				new SOAPPrimeClient(hostURL, dataFileName);
			soapClient.sendSOAPMessage();

		}
		catch(Exception e)
		{
			System.out.println(e.getMessage());
		}
	}

	/** Prints the usage. */
	private static void printUsage() {

		StringBuffer use = new StringBuffer();
		use.append("usage: java SOAPPrimeClient (options) ...\n\n");
		use.append("options:\n");
		use.append("  -url hostURL      Specify host url\n");
		use.append("                    Default host url: " + DEFAULT_HOST_URL+"\n");
		use.append("  -df datafile      Specify data file name\n");
		use.append("                    Default host file name: " + DEFAULT_DATA_FILENAME+"\n");
		use.append("  -h                This help screen.\n");
		System.err.println (use);
	}
}