package javaxml;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.math.BigInteger;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;


/**
 * @author jjoller;
 * @version 1.0
 *
 * Einfaches Servlet, welches ein Eingabe einer Integer Zahl die nchst mgliche Primzahl liefert.
 * Created on 08.08.2003
 *
 * Beispiel:
 * http://localhost:8080/JavaXML3/servlet/javaxml.PrimzahlServlet?12
 * oder
 * http://localhost:8080/JavaXML3/servlet/javaxml.PrimzahlServlet?startZahl=345&&debug=true
 * oder
 * http://localhost:8080/JavaXML3/servlet/javaxml.PrimzahlServlet?startZahl=5486
 */


public class SOAPPrimzahlServlet extends HttpServlet {

	/**
	 * GET Befehl
	 * @param Request und Response 
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {
		System.out.println("[SOAPPrimzahlServlet]doGet()");
		System.out.println("____________________________");
		System.out.println("Received request.");
		System.out.println("____________________________");

		// Headers ausgeben
		System.out.println("[SOAPPrimzahlServlet.doGet()]getHeader()");
		int ix=1;
		for(Enumeration enum = request.getHeaderNames();
			enum.hasMoreElements(); ){
			String header = (String)enum.nextElement();
			String value  = request.getHeader(header);
			System.out.println("Header "+ix+" "+ header + " = " + value);
			ix++;
		}

		System.out.println("____________________________");

		//System.out.println("[SOAPPrimzahlServlet]doGet()");
		//this.doGet(request, response);
		
		
		/**
		 * Lesen der Eingabezahl
		 */
		int iStart = 10;
		boolean bDebug = false;
		String strStart = request.getParameter("startZahl");
		String strDebug = "false";
		strDebug = request.getParameter("debug");
		if (strDebug == null)
			strDebug = "false";
		try {
			iStart = Integer.parseInt(strStart);
		} catch (Exception e) { // NumberFormat oder NullPointerException
			// somit wird iStart Standardwert verwendet 
		}
		System.out.println("[SOAPPrimzahlServlet.doGet()]Eingabe \t" + iStart);
		System.out.println("[SOAPPrimzahlServlet.doGet()]        \t" + strDebug);
		bDebug = strDebug.equalsIgnoreCase("true");

		System.out.println(
			"[SOAPPrimzahlServlet.doGet()]generieren der Antwort in XML");
		/**
		 * MIME Type
		 */
		response.setContentType("text/xml; charset=UTF-8");
		PrintWriter pw = response.getWriter();

		/**
		 * XML Header
		 */
		pw.println("<?xml version=\"1.0\"?>");
		/**
		 * Formattierung der Ausgabe mit CSS (muss ich noch erstellen!)
		 */
		//pw.print  ("<?xml-stylesheet ");
		//pw.println("type='text/css' href='styles/primzahlen.css'?>");  

		/**
		 * Bestimmen der Primzahl zu iStart ohne Debug Info-Ausgabe
		 */
		int iPrime = this.getPrime(iStart, bDebug);
		pw.println("<PrimZahlen>");

		/**
		 * Ausgabe der Primzahl 
		 */
		pw.print("  <primzahl Startzahl=\"" + iStart + "\">");
		pw.print(iPrime);

		/**
		 * End-Tags
		 */
		pw.println("</primzahl>");
		pw.println("</PrimZahlen>");
		pw.close();
	}

	/**
	 * doPost sendet die Antwort an den Client bei POST Anfragen
	 * Hier ruft die Methode einfach doGet() auf.
	 */
	public void doPost(
		HttpServletRequest request,
		HttpServletResponse response)
		throws ServletException, IOException {

		System.out.println("____________________________");
		System.out.println("[SOAPPrimzahlServlet]doPost()");

		// POST Header Zeilen ausgeben
		/*
		 * Header 1 host = localhost:5555
		 * Header 2 content-type = text/xml; charset=utf-8
		 * Header 3 content-length = 760
		 * Header 4 soapaction = "urn:javaxml"
		 */
		System.out.println("[SOAPPrimzahlServlet.doPost]getHeader() : POST Header");
		int ix=1;
		for(Enumeration enum = request.getHeaderNames();
			enum.hasMoreElements(); ){
			String header = (String)enum.nextElement();
			String value  = request.getHeader(header);
			System.out.println("Header "+ix+" "+ header + " = " + value);
			ix++;
		}

		System.out.println("____________________________");
		System.out.println("[SOAPPrimzahlServlet.doPost] SOAP Envelope");

		if(request.getContentLength() > 0)
		{
			try	{
				/*
				 * Body Entry 1 : 	<methodCall>
				 * 	<methodName>primeServer.getPrime</methodName>
				 * 	<params>
				 * 		<param>
				 * 			<value>
				 * 				<int>123</int>
				 * 			</value>
				 * 		</param>
				 * 		<param>
				 * 			<value>
				 * 				<boolean>1</boolean>
				 * 			</value>
				 * 		</param>
				 * 	</params>
				 * </methodCall>
				 */
				// Response lesen (Apache Xerces)
				DOMParser parser = new DOMParser();
				InputStream in = request.getInputStream();
				InputSource source = new InputSource(in);
				parser.parse(source);
				in.close();
	
				Document document = parser.getDocument();
				NodeList nodeList1 = document.getElementsByTagName("int");
				Node intTag = nodeList1.item(0);
				Text result1 = (Text)intTag.getFirstChild();

				NodeList nodeList = document.getElementsByTagName("boolean");
				Node booleanTag = nodeList.item(0);
				Text result2 = (Text)booleanTag.getFirstChild();

				int iStart = (new Integer(result1.getNodeValue())).intValue();
				boolean bDebug = (new Boolean(result2.getNodeValue())).booleanValue();
				System.out.println("Eingabe : Zahl \t"+iStart);
				System.out.println("          Debug\t"+bDebug);
				//
				// Generieren der Antwort
				//

				System.out.println(
					"[SOAPPrimzahlServlet.doGet()]generieren der Antwort in XML");
				response.setContentType("text/xml; charset=UTF-8");
				PrintWriter pw = response.getWriter();

				pw.println("<?xml version=\"1.0\"?>");
				pw.println("<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\">");
				pw.println("<SOAP-ENV:Body>");
				/**
				 * Formattierung der Ausgabe mit CSS (muss ich noch erstellen!)
				 */
				//pw.print  ("<?xml-stylesheet ");
				//pw.println("type='text/css' href='styles/primzahlen.css'?>");  

				/**
				 * Bestimmen der Primzahl zu iStart ohne Debug Info-Ausgabe
				 */
				int iPrime = this.getPrime(iStart, bDebug);
				pw.println("<PrimZahlen>");

				/**
				 * Ausgabe der Primzahl 
				 */
				pw.print("  <primzahl Startzahl=\"" + iStart + "\">");
				pw.print(iPrime);

				/**
				 * End-Tags
				 */
				pw.println("</primzahl>");
				pw.println("</PrimZahlen>");
				/**
				 * SOAP Body und Umschlag
				 */
				pw.println("</SOAP-ENV:Body>");
			  	pw.println("</SOAP-ENV:Envelope>");

				pw.close();


			} catch(Exception e) {
				System.out.println(e);
				e.printStackTrace();
			}
		}
	}
	public int getPrime(int i, boolean debug) {
		System.out.println(
			"[SOAPPrimzahlServlet]getPrime(" + i + "," + debug + ")");
		// solange i keine Primzahl ist, addiere 1
		while (!BigInteger.valueOf(i).isProbablePrime(20)) {
			if (debug)
				System.out.println("Primzahlsuche : " + BigInteger.valueOf(i));
			i++;
		}
		// wahrscheinlich ist i jetzt Primzahl
		// (mit Wahrscheinlichkeit 1- 1/(2^20) ~= 1 zu einer Million)
		return i;
	}

}
