package kapitel03;

/**
 * <p>Title: Java und XML</p>
 * <p>Description: Kapitel 3</p>
 * <p>Copyright: Copyright (c) 2003/4</p>
 * <p>Company: HSR</p>
 * @author J.M.Joller
 * @version 1.0
 */

import java.math.*;
import java.util.*;
import java.io.*;

public class PrimZahlXMLMitDTD {
  private final String PRIM_START_TAG = "<PrimZahlen>"; // Root
  private final String PRIM_END_TAG = "</PrimZahlen>";

  // DTD
  private final String DTD_PROLOG = "<!DOCTYPE PrimZahlen [";
  private final String DTD_ROOT = "  <!ELEMENT PrimZahlen  (primzahl)*>";
  private final String DTD_ELEMENT = "  <!ELEMENT primzahl (#PCDATA)>";
  private final String DTD_ATTLIST = "  <!ATTLIST primzahl Startzahl NMTOKEN #REQUIRED>";
  private final String DTD_EPILOG = "]>";

  private final String P_START = "<primzahl";
  private final String TAG_ABSCHLUSS = "\">";
  private final String P_ATTR = " Startzahl=\"";
  private final String P_END_TAG = "</primzahl>";

  private final static String XML_DECLARATION = "<?xml version=\"1.0\"?>";


    public int getPrime(int i, boolean debug) {
        // solange i keine Primzahl ist, addiere 1
        while (!BigInteger.valueOf(i).isProbablePrime(20)) {
          if (debug) System.out.println("Primzahlsuche : "+BigInteger.valueOf(i));
          i++;
        }
        // wahrscheinlich ist i jetzt Primzahl
        // (mit Wahrscheinlichkeit 1- 1/(2^20) ~= 1 zu einer Million)
        return i;
    }

    public static void main(String[] args) {
      PrimZahlXMLMitDTD prime = new PrimZahlXMLMitDTD();

      // von ... bis
      System.out.println("------------------------------------------------");

      // Tree : sortiert
      // Map : <Zahl, Primzahl>
      TreeMap tm = new TreeMap();
      int iPrime;
      for (int p=2040; p<2080; p++) {
        iPrime=prime.getPrime(p, false);
        //System.err.println("Nchste grssere Primzahl zu "+p+" : "+iPrime);
        tm.put(new Integer(p), new Integer(iPrime));
      }

      Object obj;
      Iterator it = tm.keySet().iterator();

      // XML Deklaration
      System.out.println(prime.XML_DECLARATION);

      // DTD
      System.out.println(prime.DTD_PROLOG);
      System.out.println(prime.DTD_ROOT);
      System.out.println(prime.DTD_ELEMENT);
      System.out.println(prime.DTD_ATTLIST);
      System.out.println(prime.DTD_EPILOG);

      // XML Daten
      System.out.println(prime.PRIM_START_TAG);
      while(it.hasNext()) {
        obj = it.next();
        System.out.println(prime.P_START+prime.P_ATTR+obj+prime.TAG_ABSCHLUSS+tm.get(obj)+prime.P_END_TAG);
      }
      System.out.println(prime.PRIM_END_TAG);

      try {
        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream("PrimMitDTD.xml"));
        // XML Deklaration
        bos.write(prime.XML_DECLARATION.getBytes());

        // DTD
        bos.write(prime.DTD_PROLOG.getBytes());
        bos.write(prime.DTD_ROOT.getBytes());
        bos.write(prime.DTD_ELEMENT.getBytes());
        bos.write(prime.DTD_ATTLIST.getBytes());
        bos.write(prime.DTD_EPILOG.getBytes());

        // XML Daten
        bos.write(prime.PRIM_START_TAG.getBytes());

        it = tm.keySet().iterator();
        while(it.hasNext()) {
          obj = it.next();
          bos.write((prime.P_START+prime.P_ATTR+obj+prime.TAG_ABSCHLUSS+tm.get(obj)+prime.P_END_TAG).getBytes());
        }
        bos.write(prime.PRIM_END_TAG.getBytes());
        bos.flush();
        bos.close();

      } catch (IOException ioe) {
        ioe.printStackTrace(System.err);
      }
    }

}